package com.qmth.themis.business.templete;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBTaskHistory;
import com.qmth.themis.business.enums.TaskStatusEnum;
import com.qmth.themis.business.enums.UploadFileEnum;
import com.qmth.themis.business.service.TBAttachmentService;
import com.qmth.themis.business.service.TBTaskHistoryService;
import com.qmth.themis.business.util.OssUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;

import java.io.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 导出任务公用
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/19
 */
public class TaskExportCommon {
    private Map<String, Object> map = null;
    private Long orgId = null;
    private Long createId = null;
    private TBTaskHistoryService tbTaskHistoryService = null;
    private OssUtil ossUtil = null;
    private Map<String, Object> ossEnv = null;
    private TBTaskHistory tbTaskHistory = null;
    private String timeFormat = null;
    private boolean exception = false;
    private TBAttachmentService tbAttachmentService = null;
    private String type = null;

    public TaskExportCommon() {

    }

    public TaskExportCommon(Map<String, Object> map) {
        this.map = map;
    }

    /**
     * 初始化
     */
    public void init() {
        SystemConfig systemConfig = SpringContextHolder.getBean(SystemConfig.class);
        Gson gson = new Gson();
        if (Objects.nonNull(this.map.get("orgId"))) {
            this.orgId = Long.parseLong(String.valueOf(this.map.get("orgId")));
        }
        this.createId = Long.parseLong(String.valueOf(this.map.get("createId")));
        this.ossUtil = SpringContextHolder.getBean(OssUtil.class);
        this.ossEnv = systemConfig.getOssEnv(UploadFileEnum.file.ordinal());
        this.type = (boolean) this.ossEnv.get("oss") ? SystemConstant.OSS : SystemConstant.LOCAL;
        Map tbTaskHistoryMap = (Map) this.map.get("tbTaskHistory");
        tbTaskHistoryMap = SystemConstant.timeTransform(tbTaskHistoryMap);
        this.tbTaskHistory = gson.fromJson(gson.toJson(tbTaskHistoryMap), TBTaskHistory.class);
        this.tbTaskHistoryService = SpringContextHolder.getBean(TBTaskHistoryService.class);
        this.tbAttachmentService = SpringContextHolder.getBean(TBAttachmentService.class);
        this.timeFormat = "yyyy-MM-dd HH:mm:ss";
    }

    /**
     * 创建excel
     *
     * @return
     */
    public File createExcel() throws IOException {
        LocalDateTime nowTime = LocalDateTime.now();
        StringJoiner stringJoiner = new StringJoiner("");
        String uploadType = String.valueOf(this.ossEnv.get(SystemConstant.UPLOAD_TYPE));
        stringJoiner.add(uploadType).add(File.separator).add(String.valueOf(nowTime.getYear()))
                .add(File.separator).add(String.valueOf(nowTime.getMonthValue()))
                .add(File.separator).add(String.valueOf(nowTime.getDayOfMonth()));
        File file = new File(stringJoiner.add(File.separator).add(String.valueOf(UUID.randomUUID()).replaceAll("-", ""))
                .add(SystemConstant.EXCEL_PREFIX).toString());
        if (!file.exists()) {
            file.getParentFile().mkdirs();
            file.createNewFile();
        }
        return file;
    }

    /**
     * oss上传
     *
     * @param file
     * @throws FileNotFoundException
     */
    public void ossUpload(File file) throws FileNotFoundException {
        boolean oss = (boolean) this.getOssEnv().get(SystemConstant.OSS);
        if (oss) {//上传至oss
            this.ossUtil.ossUpload(this.ossEnv, file.getPath(), new FileInputStream(file));
            file.delete();
        }
    }

    /**
     * 创建excel列
     *
     * @param sxssfRow
     * @param cellIndex
     * @param excelValue
     * @param style
     */
    public void createExcelCell(Row sxssfRow, int cellIndex, Object excelValue, CellStyle style) {
        Cell cell = sxssfRow.createCell(cellIndex);
        cell.setCellValue((String) excelValue);
        cell.setCellStyle(style);
    }

    /**
     * 写入txt文件
     *
     * @param path
     * @param txtStr
     * @throws IOException
     */
    public void writeExportResultTxt(String path, String txtStr) throws IOException {
        FileWriter fileWriter = null;
        try {
            path = path.substring(0, path.lastIndexOf(File.separator) + 1);
            File file = new File(path + String.valueOf(UUID.randomUUID()).replaceAll("-", "") + SystemConstant.TXT_PREFIX);
            if (!file.exists()) {
                file.getParentFile().mkdirs();
                file.createNewFile();
            }
            fileWriter = new FileWriter(file);
            fileWriter.write(txtStr);
            JSONObject json = new JSONObject();
            json.put("path", file.getPath());
            json.put("type", this.type);
            if (this.exception) {
                this.tbTaskHistory.setErrorFilePath(json.toJSONString());
            } else {
                this.tbTaskHistory.setResultFilePath(json.toJSONString());
            }
            if (Objects.equals(this.type, SystemConstant.OSS)) {
                this.ossUtil.ossUpload(this.ossEnv, file.getPath(), txtStr);
                file.delete();
            }
            tbTaskHistoryService.updateById(this.tbTaskHistory);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(fileWriter)) {
                fileWriter.flush();
                fileWriter.close();
            }
        }
    }

    /**
     * 换算进度
     *
     * @param max
     * @param min
     * @param size
     * @param txtList
     * @return
     */
    public List<String> progress(int max, int min, int size, List<String> txtList) {
        BigDecimal bigDecimal = new BigDecimal(100);
        BigDecimal progress = new BigDecimal(Double.valueOf(new BigDecimal(max).divide(new BigDecimal(size), 2, BigDecimal.ROUND_HALF_UP).multiply(bigDecimal).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue())).setScale(0, BigDecimal.ROUND_HALF_UP);
        if (progress.intValue() == 100) {
            this.tbTaskHistory.setStatus(TaskStatusEnum.finish);
            this.tbTaskHistory.setSummary("共导出了" + size + "条数据");
            this.tbTaskHistory.setFinishTime(new Date());
            txtList.add(DateUtil.format(new Date(), this.timeFormat) + "->数据导出完毕，" + this.tbTaskHistory.getSummary());
        } else {
            this.tbTaskHistory.setStatus(TaskStatusEnum.running);
            this.tbTaskHistory.setSummary("正在导出第" + min + "条至" + max + "条数据");
            txtList.add(DateUtil.format(new Date(), this.timeFormat) + "->数据导出中，" + this.tbTaskHistory.getSummary());
        }
        this.tbTaskHistory.setProgress(progress.doubleValue());
        this.tbTaskHistoryService.updateById(this.tbTaskHistory);
        return txtList;
    }

    /**
     * 异常处理
     *
     * @param min
     * @param y
     * @param e
     * @param txtList
     * @return
     */
    public List<String> exception(int min, int y, Exception e, List<String> txtList) {
        this.exception = true;
        String exceptionStr = "数据导出到第" + (min + y + 1) + "条时发生异常：" + e.getMessage();
        txtList.add(DateUtil.format(new Date(), this.timeFormat) + "->" + exceptionStr);
        this.tbTaskHistory.setSummary(exceptionStr);
        this.tbTaskHistoryService.updateById(this.tbTaskHistory);
        return txtList;
    }

    public void setException(boolean exception) {
        this.exception = exception;
    }

    public boolean isException() {
        return exception;
    }

    public String getTimeFormat() {
        return timeFormat;
    }

    public Map<String, Object> getMap() {
        return map;
    }

    public Long getOrgId() {
        return orgId;
    }

    public Long getCreateId() {
        return createId;
    }

    public TBTaskHistoryService getTbTaskHistoryService() {
        return tbTaskHistoryService;
    }

    public OssUtil getOssUtil() {
        return ossUtil;
    }

    public Map<String, Object> getOssEnv() {
        return ossEnv;
    }

    public TBTaskHistory getTbTaskHistory() {
        return tbTaskHistory;
    }

    public TBAttachmentService getTbAttachmentService() {
        return tbAttachmentService;
    }
}
