package com.qmth.themis.business.templete.impl;

import cn.hutool.core.date.DateUtil;
import com.qmth.themis.business.annotation.ExcelNote;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.RoomCodeExportDto;
import com.qmth.themis.business.service.TBExamInvigilateUserService;
import com.qmth.themis.business.templete.TaskExportCommon;
import com.qmth.themis.business.templete.TaskExportTemplete;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.*;

/**
 * @Description: 考场导出任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/20
 */
public class TaskRoomCodeExportTemplete implements TaskExportTemplete {
    private final static Logger log = LoggerFactory.getLogger(TaskRoomCodeExportTemplete.class);

    /**
     * 考场导出模版
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Result exportTask(Map<String, Object> map) throws IOException {
        log.info("开始导出考场数据");
        long start = System.currentTimeMillis();
        TaskExportCommon taskExportCommon = new TaskExportCommon(map);
        taskExportCommon.init();
        String timeFormat = taskExportCommon.getTimeFormat();
        List<String> txtList = new ArrayList();
        FileOutputStream fos = null;
        SXSSFWorkbook wb = null;
        File file = null;
        int y = 0, min = 0;
        try {
            txtList.add(DateUtil.format(new Date(), timeFormat) + "->开始准备处理导出的考场数据");
            TBExamInvigilateUserService tbExamInvigilateUserService = SpringContextHolder.getBean(TBExamInvigilateUserService.class);
            List<RoomCodeExportDto> roomCodeExportDtoList = tbExamInvigilateUserService.examInvigilateUserExport();

            //创建excel
            wb = new SXSSFWorkbook();
            Sheet sheet = wb.createSheet("考场导出");
            CellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER); // 水平居中格式
            style.setVerticalAlignment(VerticalAlignment.CENTER); //垂直居中
            Row row = sheet.createRow(0);
            Field[] fields = RoomCodeExportDto.class.getDeclaredFields();
            //绘制表头
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
                field.setAccessible(true);
                Cell cell = row.createCell(i);
                cell.setCellValue(field.getAnnotation(ExcelNote.class).value());
                cell.setCellStyle(style);
//                sheet.setColumnWidth(i, 15 * 256);
            }
            int cellIndex = 0, max = SystemConstant.MAX_EXPORT_SIZE, size = roomCodeExportDtoList.size();
            if (max >= size) {
                max = size;
            }
            while (max <= size) {
                List subList = roomCodeExportDtoList.subList(min, max);
                //绘制数据
                for (; y < subList.size(); y++) {
                    RoomCodeExportDto roomCodeExportDto = (RoomCodeExportDto) subList.get(y);
                    int firstRow = y + 1;
                    cellIndex = 0;
                    Row sxssfRow = sheet.createRow(firstRow);
                    for (Field field : fields) {
                        field.setAccessible(true);
                        taskExportCommon.createExcelCell(sxssfRow, cellIndex, field.get(roomCodeExportDto), style);
                        cellIndex++;
                    }
                }
                txtList = taskExportCommon.progress(max, min, size, txtList);
                if (max == size) {
                    break;
                }
                min = max;
                max += SystemConstant.MAX_EXPORT_SIZE;
                if (max >= size) {
                    max = size;
                }
            }
            file = taskExportCommon.createExcel();
            //写入excel并上传附件
            fos = new FileOutputStream(file);
            wb.write(fos);
            taskExportCommon.ossUpload(file);
            long end = System.currentTimeMillis();
            log.info("导出考场数据结束,============耗时============:{}秒", (end - start) / 1000);
        } catch (Exception e) {
            e.printStackTrace();
            txtList = taskExportCommon.exception(min, y, e, txtList);
            if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        } finally {
            if (Objects.nonNull(wb)) {
                wb.dispose();
            }
            if (Objects.nonNull(fos)) {
                fos.flush();
                fos.close();
            }
            //这里写入txt文件
            taskExportCommon.writeExportResultTxt(file.getPath(), txtList.toString());
        }
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }
}
