package com.qmth.themis.mq.templete.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.entity.TOeExamBreakHistory;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.TEExamStudentLogService;
import com.qmth.themis.business.service.TMRocketMessageService;
import com.qmth.themis.business.service.TOeExamBreakHistoryService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.templete.Concurrently;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.common.message.MessageExt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: mq 延时消息监听 websocket超时退出 并行消费监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/28
 */
@Service
public class WebsocketUnNormalConcurrentlyImpl implements Concurrently {
    private final static Logger log = LoggerFactory.getLogger(WebsocketUnNormalConcurrentlyImpl.class);

    @Override
    @Transactional
    public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
        RedisUtil redisUtil = SpringContextHolder.getBean(RedisUtil.class);
        TEExamStudentLogService teExamStudentLogService = SpringContextHolder.getBean(TEExamStudentLogService.class);
        TMRocketMessageService tmRocketMessageService = SpringContextHolder.getBean(TMRocketMessageService.class);
        TOeExamRecordService tOeExamRecordService = SpringContextHolder.getBean(TOeExamRecordService.class);
        TOeExamBreakHistoryService tOeExamBreakHistoryService = SpringContextHolder.getBean(TOeExamBreakHistoryService.class);
        MqDto mqDto = null;
        try {
            long threadId = Thread.currentThread().getId();
            String threadName = Thread.currentThread().getName();
            Gson gson = new Gson();
            for (MessageExt messageExt : msgs) {
                log.info(":{}-:{} websocket un normal logConsumer重试次数:{}", threadId, threadName, messageExt.getReconsumeTimes());
                String body = new String(messageExt.getBody(), Constants.CHARSET_NAME);
                log.info("body:{}", body);
                JSONObject jsonObject = JSONObject.parseObject(body);
                Map properties = (Map) jsonObject.get("properties");
                mqDto = JSONObject.toJavaObject(JSONObject.parseObject(String.valueOf(properties.get(SystemConstant.MQDTO_OBJ))), MqDto.class);
                int reconsumeTime = messageExt.getReconsumeTimes();
                if (reconsumeTime >= SystemConstant.MAXRECONSUMETIMES) {
                    //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
                    mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
                    TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                    tmRocketMessage.setProp(JacksonUtil.parseJson(tmRocketMessage.getProperties()));
                    tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                    redisUtil.delete(SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST, mqDto.getId());
                } else {
                    if (Objects.nonNull(mqDto.getAck()) && mqDto.getAck().intValue() != SystemConstant.STANDARD_ACK_TYPE && Objects.nonNull(SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST) && redisUtil.lock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId(), SystemConstant.REDIS_LOCK_MQ_TIME_OUT)) {
                        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);//表示成功处理
                        Map<String, Object> tranMap = mqDto.getProperties();
                        Long recordId = Long.parseLong(String.valueOf(tranMap.get("recordId")));
                        Date clientLastSyncTime = ExamRecordCacheUtil.getClientLastSyncTime(recordId);
                        if ((System.currentTimeMillis() - clientLastSyncTime.getTime()) / 1000 / 60 >= 2) {//大于等于当前时间，说明未重连或重登录
                            String deviceId = String.valueOf(tranMap.get("deviceId"));
                            String ip = String.valueOf(tranMap.get("ip"));
                            Long updateTime = Long.parseLong(String.valueOf(tranMap.get("updateTime")));
                            Date date = new Date();
                            date.setTime(updateTime);
                            TOeExamRecord tOeExamRecord = tOeExamRecordService.getById(recordId);
                            Integer breakCount = tOeExamRecord.getLeftBreakResumeCount();
                            if (Objects.isNull(breakCount) || breakCount <= 0) {
                                //todo 没有断点次数，则强制交卷
                                tOeExamRecord.setStatus(ExamRecordStatusEnum.finished);
                                tOeExamRecordService.updateById(tOeExamRecord);

                                //加入踢下线mq
                            } else {
                                breakCount--;
                                //增加断点记录
                                TOeExamBreakHistory tOeExamBreakHistory = new TOeExamBreakHistory(recordId, new Date(), BreakReasonEnum.NET_TIME_OUT, BreakReasonEnum.NET_TIME_OUT.name());
                                tOeExamBreakHistoryService.save(tOeExamBreakHistory);
                                //更新考试记录状态
                                tOeExamRecord.setClientCurrentIp(ip);
                                tOeExamRecord.setClientWebsocketId(deviceId);
                                tOeExamRecord.setClientWebsocketStatus(WebsocketStatusEnum.UN_ONLINE.ordinal());
                                tOeExamRecord.setClientLastSyncTime(date);
                                tOeExamRecord.setLastBreakId(tOeExamBreakHistory.getId());
                                tOeExamRecord.setLastBreakTime(tOeExamBreakHistory.getBreakTime());
                                tOeExamRecord.setLeftBreakResumeCount(breakCount);
                                tOeExamRecord.setStatus(ExamRecordStatusEnum.break_off);
                                tOeExamRecordService.updateById(tOeExamRecord);
                            }
                        }
                    } else {
                        mqDto.setAck(SystemConstant.INDIVIDUAL_ACK_TYPE);//表示考生已重新登录
                    }
                    log.info(":{}-:{} 插入用户轨迹日志", threadId, threadName);
                    teExamStudentLogService.saveStudentLogInfo(mqDto.getTimestamp(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
                    TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                    tmRocketMessage.setProp(JacksonUtil.parseJson(tmRocketMessage.getProperties()));
                    tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                    redisUtil.delete(SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST, mqDto.getId());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        } finally {
            if (Objects.nonNull(mqDto)) {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId());
            }
        }
        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
    }
}
